import serial
import time
import threading

import http.client
import urllib.request
import urllib.parse
import json

# Parametros del dispositivo:

api_url_tag = "http://192.168.2.55:8000/enviodatoscronometraje/"
api_url_estado = "http://192.168.2.55:8000/enviodatosestado/"
device = "device-12"
api_key = "2209bfa1f1a65c24044f32ea492c9c147fde1099c0bb7d7c4e08c6c0abb0c568d46f8774ed992aa0"

path_log_file = "/home/pi/sced/log.txt"
path_tags_file = "/home/pi/sced/tags.txt"

########

def desempaquetar_paquete_tag_inventory(paquete):
	global tags_leidos_antena
	freq_antenna = paquete[0]
	pc = int.from_bytes(paquete[1:3], byteorder='big')
	epc = "-".join(str(i) for i in paquete[3:len(paquete)-1])
	rssi = paquete[len(paquete)-1]

	# Se anade el tag a la lista de leidos si no estaba antes
	# y si el RRSI tiene un valor mayor que 2
	if epc not in tags_leidos_antena and rssi > 2:
		tags_leidos_antena[epc] = time.time()


def desempaquetar_paquete_work_antenna(ser, paquete):
	global error_antena
	error_code = int.from_bytes(paquete, byteorder='big')
	if error_code == 16:
		error_antena = 0
		ser.write(b"\xA0\x04\x01\x89\x01\xD1")
	else:
		error_antena = 1


def enviar_paquete_servidor(data, api_url):
	binary_data = json.dumps(data).encode('utf-8')
	header = {
	    "Content-Type": "application/json",
	    "Api-Key": api_key,
	    "User-Agent": device,
	}
	try:
		req = urllib.request.Request(api_url, binary_data, header)
		f = urllib.request.urlopen(req, timeout=2)
		#response_data = json.loads(f.read().decode('utf-8'))
		#print(json.dumps(response_data, indent=4, sort_keys=True))
		return True
	except:
		return False


def comunicar_estado_servidor(error_antena):
	global api_url_estado
	tiempo_actual = time.time()
	estado = not bool(error_antena)
	data = {
		"estado": estado,
		"estado_tiempo": tiempo_actual,
	}
	if enviar_paquete_servidor(data, api_url_estado):
		return True
	return False


def almacenar_estado_local(error_antena):
	global path_log_file
	tiempo_actual = time.time()
	if error_antena:
		estado_actual = "La antena no funciona"
	else:
		estado_actual = "La antena funciona correctamente"
	
	with open(path_log_file, 'a') as file:
		file.write(str(tiempo_actual) + ' : ' + str(estado_actual) + '\n')


def enviar_tag_servidor(tag, tag_time):
	global api_url_tag
	data = {
		"etiqueta_rfid": tag,
		"tiempo_medido": tag_time,
	}
	print(tag)
	if enviar_paquete_servidor(data, api_url_tag):
		return True
	return False


def almacenar_tag_local(tag, tag_time):
	global path_tags_file
	with open(path_tags_file, 'a') as file:
		file.write(str(tag) + ' : ' + str(tag_time) + '\n')

def reenviar_tags_servidor():
	global path_tags_file
	# Se obtienen todos los tags pendientes y se vacia el fichero
	with open(path_tags_file, 'r+') as file:
		file_lines = file.readlines()
		file.truncate(0)

	# Se ponen los tags en un diccionario
	tags_reenviar = {}
	for line in reversed(file_lines):
		[tag, tag_time] = line.split(' : ')
		tags_reenviar[tag] = tag_time

	# Se intenta enviar cada tag, si falla el reenvio se almacenan de nuevo en fichero
	for tag, tag_time in tags_reenviar.items():
		if not enviar_tag_servidor(tag, tag_time):
			almacenar_tag_local(tag, tag_time)


def antena_thread():

	ser = serial.Serial(
		port='/dev/ttyS0',
		baudrate=115200,
		bytesize=serial.EIGHTBITS,
		parity=serial.PARITY_NONE,
		stopbits=serial.STOPBITS_ONE,
	)


	while True:

		if ser.inWaiting() > 0:
			if ser.read() == b"\xa0":
				packet_size = int.from_bytes(ser.read(), byteorder='big')
				if ser.inWaiting() >= packet_size:
				
					packet_content = ser.read(packet_size)
					packet_addr = packet_content[0]
					packet_tipe = packet_content[1]
					packet_data = packet_content[2:packet_size-1]
					packet_check = packet_content[packet_size-1]
					
					if packet_tipe == 137:
						desempaquetar_paquete_tag_inventory(packet_data)
						
					elif packet_tipe == 116:
						desempaquetar_paquete_work_antenna(ser, packet_data)

		else:
			ser.write(b"\xa0\x04\x01\x74\x00\xe7")
			time.sleep(0.04)


def gestion_thread():
	global tags_leidos_antena
	global tags_gestionados
	global error_antena
	global ultima_comunicacion_estado

	while True:

		# Si hay errores intentamos comunicarlo al servidor
		# Si la conexion falla, lo almacenamos en local
		if time.time() - ultima_comunicacion_estado >= 60:
			ultima_comunicacion_estado = time.time()
			if comunicar_estado_servidor(error_antena):
				reenviar_tags_servidor()
			else:
				almacenar_estado_local(error_antena)

			

		# Se limpian los tags gestionados antiguos
		# Si han pasado mas de 5 segundos se consideran antiguos
		tags_gestionados_iterar = tags_gestionados.copy()
		for tag, tag_time in tags_gestionados_iterar.items():
			if time.time() - tag_time > 5:
				del tags_gestionados[tag]
			
		# se emplea otra variable diferente porque el diccionario tags_leidos_antena
		# podria cambiar su tamano durante la iteraccion del bucle
		tags_iterables = tags_leidos_antena.copy()
		tags_leidos_antena = {}
		for tag, tag_time in tags_iterables.items():

			# Si hay un tag que no teniamos intentamos enviarlo al serivor
			# Si falla el envio, lo almacenamos en un fichero local
			if tag not in tags_gestionados:
				tags_gestionados[tag] = tag_time
				if not enviar_tag_servidor(tag, tag_time):
					almacenar_tag_local(tag, tag_time)




tags_leidos_antena = {}
tags_gestionados = {}
error_antena = 1 # De inicio suponemos que la antena no funciona
ultima_comunicacion_estado = 0

hilo_antena = threading.Thread(target=antena_thread)
hilo_gestion = threading.Thread(target=gestion_thread)
hilo_antena.start()
hilo_gestion.start()