from django import forms

from .models import *

ZONA_HORARIA_CHOICES = [
	('UTC', 'UTC'),
	('UTC + 1', 'UTC + 1'),
	('UTC - 1', 'UTC - 1'),
	('UTC + 2', 'UTC + 2'),
	('UTC - 2', 'UTC - 2'),
	('UTC + 3', 'UTC + 3'),
	('UTC - 3', 'UTC - 3'),
	('UTC + 4', 'UTC + 4'),
	('UTC - 4', 'UTC - 4'),
	('UTC + 5', 'UTC + 5'),
	('UTC - 5', 'UTC - 5'),
	('UTC + 6', 'UTC + 6'),
	('UTC - 6', 'UTC - 6'),
	('UTC + 7', 'UTC + 7'),
	('UTC - 7', 'UTC - 7'),
	('UTC + 8', 'UTC + 8'),
	('UTC - 8', 'UTC - 8'),
	('UTC + 9', 'UTC + 9'),
	('UTC - 9', 'UTC - 9'),
	('UTC + 10', 'UTC + 10'),
	('UTC - 10', 'UTC - 10'),
	('UTC + 11', 'UTC + 11'),
	('UTC - 11', 'UTC - 11'),
	('UTC + 12', 'UTC + 12'),
	('UTC - 12', 'UTC - 12'),
]


class EquipoDeCronometrajeFormulario(forms.ModelForm):
	
	class Meta:
		model = EquipoDeCronometraje
		fields = [
			'nombre',
		]
		widgets = {
			'nombre': forms.TextInput(attrs={'class':'form-control'}),
		}

class EventoDeportivoFormulario(forms.ModelForm):
	class Meta:
		model = EventoDeportivo
		fields = [
			'nombre',
			'descripcion',
			'fecha_evento',
			'zona_horaria',
			'lugar',
			'tiempo_minimo_medicion',
		]
		widgets = {
			'nombre': forms.TextInput(attrs={'class':'form-control'}),
			'descripcion': forms.Textarea(attrs={'cols': 80, 'rows': 4, 'class':'form-control'}),
			'fecha_evento': forms.DateTimeInput(attrs={'class':'form-control'}),
			'zona_horaria': forms.Select(attrs={'class':'form-control'}, choices=ZONA_HORARIA_CHOICES),
			'lugar': forms.TextInput(attrs={'class':'form-control'}),
			'tiempo_minimo_medicion': forms.NumberInput(attrs={'class':'form-control'}),
			}

class RecorridoFormulario(forms.ModelForm):
	class Meta:
		model = Recorrido
		fields = [
			'nombre',
			'descripcion',
			'numero_de_vueltas',
			'tiempo_minimo_vuelta',
		]
		widgets = {
			'nombre': forms.TextInput(attrs={'class':'form-control'}),
			'descripcion': forms.Textarea(attrs={'cols': 80, 'rows': 4, 'class':'form-control'}),
			'numero_de_vueltas': forms.NumberInput(attrs={'class':'form-control'}),
			'tiempo_minimo_vuelta': forms.NumberInput(attrs={'class':'form-control'}),
		}

class PuntoDeControlFormulario(forms.ModelForm):
	class Meta:
		model = PuntoDeControl
		fields = [
			'nombre',
			'punto_kilometrico',
			'equipo_de_cronometraje',
		]
		widgets = {
			'nombre': forms.TextInput(attrs={'class':'form-control'}),
			'punto_kilometrico': forms.NumberInput(attrs={'class':'form-control'}),
			'equipo_de_cronometraje': forms.Select(attrs={'class':'form-control'}),
		}

	def __init__(self, recorrido_id, punto_de_control_id, *kargs, **kwargs):
		super(PuntoDeControlFormulario, self).__init__(*kargs, **kwargs)
		self.recorrido_id = recorrido_id
		self.punto_de_control_id = punto_de_control_id
		self.fields['equipo_de_cronometraje'].empty_label = "Selecciona un equipo de cronometraje"

	def clean_equipo_de_cronometraje(self):
		print(self.recorrido_id)
		puntos_de_control = PuntoDeControl.objects.filter(recorrido_id=self.recorrido_id)
		equipo_de_cronometraje = self.cleaned_data['equipo_de_cronometraje']
		equipos_en_uso = []
		for punto_de_control in puntos_de_control:
			if punto_de_control.id != self.punto_de_control_id:
				equipos_en_uso.append(punto_de_control.equipo_de_cronometraje)
		
		if equipo_de_cronometraje is not None and equipo_de_cronometraje in equipos_en_uso:
			raise forms.ValidationError("El equipo de cronometraje %s está configurado en otro punto de control del recorrido. Por favor, asigna uno diferente" % equipo_de_cronometraje)
		
		return equipo_de_cronometraje


class CategoriaFormulario(forms.ModelForm):
	class Meta:
		model = Categoria
		fields = [
			'recorrido',
			'nombre',
			'descripcion',
		]
		widgets = {
			'recorrido': forms.Select(attrs={'class':'form-control'}),
			'nombre' : forms.TextInput(attrs={'class':'form-control'}),
			'descripcion': forms.Textarea(attrs={'cols': 80, 'rows': 4, 'class':'form-control'}),
		}

	def __init__(self, evento_id, *kargs, **kwargs):
		super(CategoriaFormulario, self).__init__(*kargs, **kwargs)
		self.fields['recorrido'].queryset = Recorrido.objects.filter(evento_id=evento_id)
		self.fields['recorrido'].empty_label = "Selecciona un recorrido"


class ParticipanteFormulario(forms.ModelForm):
	class Meta:
		model = Participante
		fields = [
			'categoria',
			'nombre_completo',
			'dorsal',
			'etiqueta_rfid',
		]
		widgets = {
			'categoria': forms.Select(attrs={'class':'form-control'}),
			'nombre_completo' : forms.TextInput(attrs={'class':'form-control'}),
			'dorsal': forms.NumberInput(attrs={'class':'form-control'}),
			'etiqueta_rfid' : forms.TextInput(attrs={'class':'form-control'}),
		}

	def __init__(self, participante_id, evento_id, *kargs, **kwargs):
		super(ParticipanteFormulario, self).__init__(*kargs, **kwargs)
		self.fields['categoria'].queryset = Categoria.objects.filter(evento_id=evento_id)
		self.participante_id = participante_id
		self.evento_id = evento_id
		self.fields['categoria'].empty_label = "Selecciona una categoría"

	
	def clean_categoria(self):
		categoria = self.cleaned_data['categoria']
		return categoria

	def clean_dorsal(self):
		dorsal = self.cleaned_data['dorsal']
		
		otros_participantes = Participante.objects.filter(evento_id=self.evento_id)
		dorsales_en_uso = []
		for participante in otros_participantes:
			if participante.id != self.participante_id:
				dorsales_en_uso.append(participante.dorsal)
		
		if dorsal is not None and dorsal in dorsales_en_uso:
			raise forms.ValidationError("El dorsal %s ya está asignado a otro participante del evento. Por favor, asigna uno diferente" % dorsal)
		
		return dorsal

	def clean_etiqueta_rfid(self):
		etiqueta_rfid = self.cleaned_data['etiqueta_rfid']

		otros_participantes = Participante.objects.filter(evento_id=self.evento_id)
		etiquetas_en_uso = []
		for participante in otros_participantes:
			if participante.id != self.participante_id:
				etiquetas_en_uso.append(participante.etiqueta_rfid)
		
		if etiqueta_rfid is not None and etiqueta_rfid in etiquetas_en_uso:
			raise forms.ValidationError("La etiqueta RFID %s ya está asignada a otro participante del evento. Por favor, asigna una diferente" % etiqueta_rfid)
		
		return etiqueta_rfid

