from django.db import models
from django.urls import reverse

from django.utils import timezone

import time
from datetime import timedelta


class EquipoDeCronometraje(models.Model):
	nombre = models.CharField(max_length=200)
	api_key = models.CharField(max_length=40)
	estado = models.BooleanField(default=False) # True si el lector correctamente, False si hay errores
	estado_fecha = models.DateTimeField(null=True, blank=True)
	modo_etiqueta_participante_fecha = models.DateTimeField(null=True, blank=True)
	creado_fecha = models.DateTimeField(auto_now_add=True)
	actualizado_fecha = models.DateTimeField(auto_now=True)

	def __str__(self):
		return self.nombre

	def get_absolute_url(self):
		return reverse('cronoapp:admin_equipo_detalle', kwargs={'pk': self.pk})

	def obtener_estado(self):
		if self.estado_fecha is None:
			return "Desconectado"
		elif self.estado and time.time() - self.estado_fecha.timestamp() < 180:
			return "Operativo"
		elif not self.estado and time.time() - self.estado_fecha.timestamp() < 180:
			return "No Operativo"
		else:
			return "Desconectado"

	def obtener_ultima_conexion(self):
		if self.estado_fecha is None:
			return "Este equipo no se ha conectado nunca"
		else:
			seconds = float(time.time() - self.estado_fecha.timestamp())
			if seconds > 0:
				minutes, seconds = divmod(seconds, 60)
				hours, minutes = divmod(minutes, 60)
				days, hours = divmod(hours, 24)
				if days > 0:
					return "Hace %s días, %s horas, %s minutos y %s segundos" % (int(days), int(hours), int(minutes), int(seconds))
				elif hours > 0:
					return "Hace %s horas, %s minutos y %s segundos" % (int(hours), int(minutes), int(seconds))
				elif minutes > 0:
					return "Hace %s minutos y %s segundos" % (int(minutes), int(seconds))
				else:
					return "Hace %s segundos" % int(seconds)
			else:
				return "Hace 0 segundos"
			

class EventoDeportivo(models.Model):
	nombre = models.CharField(max_length=200)
	descripcion = models.CharField(max_length=2000, null=True, blank=True)
	fecha_evento = models.DateTimeField()
	zona_horaria = models.CharField(max_length=20)
	lugar = models.CharField(max_length=200, null=True, blank=True)
	comenzado = models.BooleanField(default=False) # Si ya ha comenzado la carrera
	finalizado = models.BooleanField(default=False) # Si ha sido finalizada por parte de la organizacion
	instante_comienzo = models.DateTimeField(null=True, blank=True)
	tiempo_minimo_medicion = models.PositiveSmallIntegerField(default=2) # tiempo minimo que debe pasar 
	# desde que comienza el evento hasta que se pueden empezar a tomar medidas (en minutos)
	#imagen_cartel = models.ImageField(null=True, blank=True)
	creado_fecha = models.DateTimeField(auto_now_add=True)
	actualizado_fecha = models.DateTimeField(auto_now=True)

	def __str__(self):
		return self.nombre

	def get_absolute_url(self):
		return reverse('cronoapp:admin_evento_detalle', kwargs={'pk': self.pk})

	def obtener_estado_evento(self):
		if self.comenzado and self.finalizado:
			return "Finalizado"
		elif self.comenzado:
			return "En directo"
		else:
			return "Evento por comenzar"

	def obtener_tiempo_transcurrido(self):
		if self.instante_comienzo is None:
			return "El evento todavía no ha comenzado"
		else:
			seconds = float(time.time() - self.instante_comienzo.timestamp())
			if seconds > 0:
				minutes, seconds = divmod(seconds, 60)
				hours, minutes = divmod(minutes, 60)
				days, hours = divmod(hours, 24)
				if days > 0:
					return "%s días, %s horas, %s minutos y %s segundos" % (int(days), int(hours), int(minutes), int(seconds))
				elif hours > 0:
					return "%s horas, %s minutos y %s segundos" % (int(hours), int(minutes), int(seconds))
				elif minutes > 0:
					return "%s minutos y %s segundos" % (int(minutes), int(seconds))
				else:
					return "%s segundos" % int(seconds)
			else:
				return "0 segundos"

	def comenzar_evento(self):
		self.comenzado = True
		self.instante_comienzo = timezone.now()
		self.save()

	def finalizar_evento(self):
		self.finalizado = True
		self.save()

class Recorrido(models.Model):
	evento = models.ForeignKey(EventoDeportivo, on_delete=models.CASCADE)
	nombre = models.CharField(max_length=200)
	descripcion = models.CharField(max_length=2000, null=True, blank=True)
	numero_de_vueltas = models.PositiveSmallIntegerField(default=1)
	tiempo_minimo_vuelta = models.PositiveSmallIntegerField(default=2)
	creado_fecha = models.DateTimeField(auto_now_add=True)
	actualizado_fecha = models.DateTimeField(auto_now=True)

	def __str__(self):
		return self.nombre

class PuntoDeControl(models.Model):
	recorrido = models.ForeignKey(Recorrido, on_delete=models.CASCADE)
	equipo_de_cronometraje = models.ForeignKey(EquipoDeCronometraje, on_delete=models.CASCADE)
	nombre = models.CharField(max_length=200)
	punto_kilometrico = models.FloatField()
	# El orden en el que van ordenados los Puntos de control se determina en funcion del punto kilometrico
	creado_fecha = models.DateTimeField(auto_now_add=True)
	actualizado_fecha = models.DateTimeField(auto_now=True)

	def __str__(self):
		return self.nombre

	class Meta:
		ordering = ['punto_kilometrico']

class Categoria(models.Model):
	evento = models.ForeignKey(EventoDeportivo, on_delete=models.CASCADE)
	recorrido = models.ForeignKey(Recorrido, on_delete=models.CASCADE)
	nombre = models.CharField(max_length=200)
	descripcion = models.CharField(max_length=2000, null=True, blank=True)
	creado_fecha = models.DateTimeField(auto_now_add=True)
	actualizado_fecha = models.DateTimeField(auto_now=True)

	def __str__(self):
		return self.nombre

class Participante(models.Model):
	evento = models.ForeignKey(EventoDeportivo, on_delete=models.CASCADE)
	categoria = models.ForeignKey(Categoria, on_delete=models.CASCADE)
	nombre_completo = models.CharField(max_length=300)
	dorsal = models.PositiveIntegerField(null=True, blank=True)
	etiqueta_rfid = models.CharField(max_length=100, null=True, blank=True)
	creado_fecha = models.DateTimeField(auto_now_add=True)
	actualizado_fecha = models.DateTimeField(auto_now=True)

	def __str__(self):
		return self.nombre_completo

	class Meta:
		ordering = ['dorsal', 'nombre_completo']


class EnvioCronometraje(models.Model):
	evento = models.ForeignKey(EventoDeportivo, on_delete=models.CASCADE)
	categoria = models.ForeignKey(Categoria, on_delete=models.CASCADE)
	punto_de_control = models.ForeignKey(PuntoDeControl, on_delete=models.CASCADE)
	participante = models.ForeignKey(Participante, on_delete=models.CASCADE)
	tiempo_medido = models.DateTimeField()
	vuelta = models.PositiveSmallIntegerField(default=1)
	creado_fecha = models.DateTimeField(auto_now_add=True)
	actualizado_fecha = models.DateTimeField(auto_now=True)


class EnvioEtiquetaParticipante(models.Model):
	etiqueta_rfid = models.CharField(max_length=100)
	creado_fecha = models.DateTimeField(auto_now_add=True)
	equipo = models.ForeignKey(EquipoDeCronometraje, on_delete=models.CASCADE)