from django.urls import path, include

from django.contrib.auth import views as auth_views
from . import views

app_name = 'cronoapp'
urlpatterns = [


	# URLs pertenecientes al sitio web publico

	path('', views.InicioView.as_view(), name='inicio'),
	path('evento/', views.EventoListadoView.as_view(), name='evento_listado'),
	path('evento/<int:pk>/', views.EventoDetalleView.as_view(), name='evento_detalle'),
	path('evento/<int:pk>/seguimiento/', views.SeguimientoView.as_view(), name='seguimiento'),
	path('evento/<int:pk>/seguimiento/<int:id_categoria>/', views.SeguimientoView.as_view(), name='seguimiento_categoria'),


	# URLs para el envío de informacion de cronometraje y de estado

	path('enviodatoscronometraje/', views.EnvioDatosCronometrajeView.as_view(), name='envio_datos_crono'),
	path('enviodatosestado/', views.EnvioDatosEstadoView.as_view(), name='envio_datos_estado'),


	# URLs de Login, logout, rest pass,...
	path('usuario/login/', auth_views.LoginView.as_view(template_name='cronoapp/usuario/login.html'), name='usuario_login'),
	path('usuario/logout/', auth_views.LogoutView.as_view(template_name='cronoapp/usuario/logout.html', next_page='cronoapp:usuario_login'), name='usuario_logout'),

	
	# URLs pertenecientes al panel de administracion de la aplicacion

	path('paneladmin/', views.AdminInicioView.as_view(), name='admin_inicio'),

	path('paneladmin/equipo/', views.AdminEquipoListadoView.as_view(), name='admin_equipo_listado'),
	path('paneladmin/equipo/anadir/', views.AdminEquipoAnadirView.as_view(), name='admin_equipo_anadir'),
	path('paneladmin/equipo/<int:pk>/', views.AdminEquipoDetalleView.as_view(), name='admin_equipo_detalle'),
	path('paneladmin/equipo/<int:pk>/editar/', views.AdminEquipoEditarView.as_view(), name='admin_equipo_editar'),
	path('paneladmin/equipo/<int:pk>/eliminar/', views.AdminEquipoEliminarView.as_view(), name='admin_equipo_eliminar'),
	
	path('paneladmin/equipo/operativosjson/', views.AdminObtenerEquiposCronometraje.as_view(), name='admin_equipos_operativos_json'),
	path('paneladmin/equipo/actualizarmodo/', views.AdminActualizarEquipoModoFecha.as_view(), name='admin_equipo_actualizar_modo'),
	path('paneladmin/equipo/obteneretiquetaparticipante/', views.AdminObtenerEtiquetaParticipante.as_view(), name='admin_obtener_etiqueta_participante'),

	path('paneladmin/evento/', views.AdminEventoListadoView.as_view(), name='admin_evento_listado'),
	path('paneladmin/evento/anadir/', views.AdminEventoAnadirView.as_view(), name='admin_evento_anadir'),
	path('paneladmin/evento/<int:pk>/', views.AdminEventoDetalleView.as_view(), name='admin_evento_detalle'),
	path('paneladmin/evento/<int:pk>/editar/', views.AdminEventoEditarView.as_view(), name='admin_evento_editar'),
	path('paneladmin/evento/<int:pk>/eliminar/', views.AdminEventoEliminarView.as_view(), name='admin_evento_eliminar'),
	path('paneladmin/evento/<int:pk>/comenzar/', views.AdminEventoComenzarView.as_view(), name='admin_evento_comenzar'),
	path('paneladmin/evento/<int:pk>/finalizar/', views.AdminEventoFinalizarView.as_view(), name='admin_evento_finalizar'),

	path('paneladmin/evento/<int:evento_id>/recorrido/', views.AdminRecorridoListadoView.as_view(), name='admin_recorrido_listado'),
	path('paneladmin/evento/<int:evento_id>/recorrido/anadir/', views.AdminRecorridoAnadirView.as_view(), name='admin_recorrido_anadir'),
	path('paneladmin/evento/<int:evento_id>/recorrido/<int:pk>/', views.AdminRecorridoDetalleView.as_view(), name='admin_recorrido_detalle'),
	path('paneladmin/evento/<int:evento_id>/recorrido/<int:pk>/editar/', views.AdminRecorridoEditarView.as_view(), name='admin_recorrido_editar'),
	path('paneladmin/evento/<int:evento_id>/recorrido/<int:pk>/eliminar/', views.AdminRecorridoEliminarView.as_view(), name='admin_recorrido_eliminar'),

	path('paneladmin/evento/<int:evento_id>/recorrido/<int:recorrido_id>/puntocontrol/anadir/', views.AdminPuntoDeControlAnadirView.as_view(), name='admin_puntocontrol_anadir'),
	path('paneladmin/evento/<int:evento_id>/recorrido/<int:recorrido_id>/puntocontrol/<int:pk>/', views.AdminPuntoDeControlDetalleView.as_view(), name='admin_puntocontrol_detalle'),
	path('paneladmin/evento/<int:evento_id>/recorrido/<int:recorrido_id>/puntocontrol/<int:pk>/editar/', views.AdminPuntoDeControlEditarView.as_view(), name='admin_puntocontrol_editar'),
	path('paneladmin/evento/<int:evento_id>/recorrido/<int:recorrido_id>/puntocontrol/<int:pk>/eliminar/', views.AdminPuntoDeControlEliminarView.as_view(), name='admin_puntocontrol_eliminar'),

	path('paneladmin/evento/<int:evento_id>/categoria/', views.AdminCategoriaListadoView.as_view(), name='admin_categoria_listado'),
	path('paneladmin/evento/<int:evento_id>/categoria/anadir/', views.AdminCategoriaAnadirView.as_view(), name='admin_categoria_anadir'),
	path('paneladmin/evento/<int:evento_id>/categoria/<int:pk>/', views.AdminCategoriaDetalleView.as_view(), name='admin_categoria_detalle'),
	path('paneladmin/evento/<int:evento_id>/categoria/<int:pk>/editar/', views.AdminCategoriaEditarView.as_view(), name='admin_categoria_editar'),
	path('paneladmin/evento/<int:evento_id>/categoria/<int:pk>/eliminar/', views.AdminCategoriaEliminarView.as_view(), name='admin_categoria_eliminar'),

	path('paneladmin/evento/<int:evento_id>/participante/', views.AdminParticipanteListadoView.as_view(), name='admin_participante_listado'),
	path('paneladmin/evento/<int:evento_id>/participante/anadir/', views.AdminParticipanteAnadirView.as_view(), name='admin_participante_anadir'),
	path('paneladmin/evento/<int:evento_id>/participante/<int:pk>/', views.AdminParticipanteDetalleView.as_view(), name='admin_participante_detalle'),
	path('paneladmin/evento/<int:evento_id>/participante/<int:pk>/editar/', views.AdminParticipanteEditarView.as_view(), name='admin_participante_editar'),
	path('paneladmin/evento/<int:evento_id>/participante/<int:pk>/eliminar/', views.AdminParticipanteEliminarView.as_view(), name='admin_participante_eliminar'),

]