from django.views import View
from django.views.generic import TemplateView
from django.views.generic.list import ListView
from django.views.generic.detail import DetailView
from django.views.generic.edit import CreateView, UpdateView, DeleteView
from django.urls import reverse_lazy
from django.contrib.auth.mixins import LoginRequiredMixin
from django.contrib import messages
from django.core import serializers as core_serializers

from rest_framework.views import APIView
from django.http import HttpResponse, JsonResponse, HttpResponseRedirect
from django.shortcuts import get_object_or_404
from django.utils import timezone

from datetime import datetime, timedelta
import time

from binascii import hexlify
from os import urandom
import json

from .models import *
from .forms import *
from .serializers import *


class InicioView(TemplateView):
	template_name = "cronoapp/inicio.html"

class EventoListadoView(TemplateView):
	template_name = "cronoapp/evento_listado.html"

	def get_context_data(self, **kargs):
		context = super(EventoListadoView, self).get_context_data(**kargs)
		context['eventos'] = EventoDeportivo.objects.all()
		return context

class EventoDetalleView(TemplateView):
	template_name = "cronoapp/evento_detalle.html"

	def get_context_data(self, **kargs):
		context = super(EventoDetalleView, self).get_context_data(**kargs)
		context['evento'] = EventoDeportivo.objects.get(pk=kargs['pk'])
		return context

class SeguimientoView(TemplateView):
	template_name = "cronoapp/seguimiento.html"

	def get_context_data(self, **kargs):
		context = super(SeguimientoView, self).get_context_data(**kargs)
		evento = EventoDeportivo.objects.get(pk=kargs['pk'])

		try:
			categoria = Categoria.objects.get(pk=kargs['id_categoria'])
		except:
			categoria = Categoria.objects.filter(evento_id=kargs['pk'])[0]

		context['evento'] = evento
		context['categoria'] = categoria

		if evento.comenzado == True:

			context['clasificaciones'] = {}

			puntos_de_control = categoria.recorrido.puntodecontrol_set.all().order_by('punto_kilometrico')
			for vuelta in range(1, categoria.recorrido.numero_de_vueltas+1):
				for punto_de_control in puntos_de_control:
					if categoria.recorrido.numero_de_vueltas > 1:
						nombre_a_mostrar = punto_de_control.nombre + " , vuelta " + str(vuelta)
					else:
						nombre_a_mostrar = punto_de_control.nombre
					context['clasificaciones'][nombre_a_mostrar] = self.generar_clasificacion_punto_control(categoria, punto_de_control, vuelta, evento.instante_comienzo)
		return context

	def generar_clasificacion_punto_control(self, categoria, punto_de_control, vuelta, instante_comienzo):
		try:
			envios_crono = EnvioCronometraje.objects.filter(categoria_id = categoria.id, punto_de_control_id=punto_de_control.id, vuelta=vuelta)
			clasificacion = []
			for envio_crono in envios_crono:
				tiempo_medido = envio_crono.tiempo_medido - instante_comienzo
				envio_crono.tiempo_medido = timedelta(days=tiempo_medido.days, seconds=tiempo_medido.seconds, microseconds=0)
				clasificacion.append(envio_crono)
				clasificacion.sort(key=lambda x: x.tiempo_medido, reverse=False)
			return clasificacion
		except:
			return []



"""---------------------------------------------------------------------------------
	
	Vistas pertenecientes a los envios de los equipos de cronometraje

-----------------------------------------------------------------------------------"""

class EnvioDatosCronometrajeView(APIView):

	evento = None
	equipo = None
	punto_de_control = None
	participante = None
	tiempo_medido = None
	vuelta = 1
	

	def post(self, request, format=None):
		try:
			self.comprobar_api_key(request)

		except:
			return JsonResponse({"Resultado del envio": "NO PERMITIDO"})

		# Si se esta leyendo una etiqueta para anadir/editar un participante
		if (float(time.time() - self.equipo.modo_etiqueta_participante_fecha.timestamp()) <= 60):
			datos = json.loads(request.body.decode("utf-8"))
			EnvioEtiquetaParticipante.objects.create(
				etiqueta_rfid = datos['etiqueta_rfid'],
				equipo_id = self.equipo.id,
			)
			return JsonResponse({"Resultado del envio": "OK"})

		# Si el envio de cronometraje corresponde a un evento deportivo
		else:
			try:
				if self.comprobar_evento(request) and \
					self.comprobar_participante(request) and \
					self.comprobar_tiempo_minimo_medicion() and \
					self.establecer_punto_control() and \
					self.comprobar_vuelta():
						EnvioCronometraje.objects.create(
							evento_id = self.evento,
							categoria_id = self.participante.categoria.id,
							punto_de_control_id = self.punto_de_control.id,
							participante_id = self.participante.id,
							tiempo_medido = self.tiempo_medido,
							vuelta = self.vuelta,
						)
						return JsonResponse({"Resultado del envio": "OK"})

				else:
					return JsonResponse({"Resultado del envio": "NO PERMITIDO"})

			except:
				return JsonResponse({"Resultado del envio": "NO PERMITIDO"})

		


	"""
	Comprobar que el envio procede de un equipo de cronometraje válido,
	basandonos en su id y su api_key
	"""
	def comprobar_api_key(self, request):
		api_key = str(request.META.get('HTTP_API_KEY', ''))
		device_id = int(str(request.META.get('HTTP_USER_AGENT', ''))[7:])

		self.equipo = EquipoDeCronometraje.objects.get(pk=device_id)
		if api_key == self.equipo.api_key:
			return True
		return False


	"""
	Comprobar que este equipo está asociado al menos a un recorrido y a un
	punto de control de un evento que se esté celebrando en estos momentos
	"""
	def comprobar_evento(self, request):
		device_id = int(str(request.META.get('HTTP_USER_AGENT', ''))[7:])
		eventos_en_directo = EventoDeportivo.objects.filter(comenzado=True, finalizado=False)
		if eventos_en_directo.count() > 0:
			for evento in eventos_en_directo:
				recorridos = Recorrido.objects.filter(evento=evento.id)
				if recorridos.count() > 0:
					for recorrido in recorridos:
						puntos_de_control = PuntoDeControl.objects.filter(recorrido=recorrido.id)
						if puntos_de_control.count() > 0:
							for punto_de_control in puntos_de_control:
								if int(punto_de_control.equipo_de_cronometraje.id) == device_id:
									self.evento = int(evento.id)
									return True
		return False


	"""
	Comprobar que la etiqueta RFID leida está asignada a un participante
	del evento deportivo
	"""
	def comprobar_participante(self, request):
		datos = json.loads(request.body.decode("utf-8"))
		participante = Participante.objects.get(evento=self.evento, etiqueta_rfid=datos['etiqueta_rfid'])
		if participante:
			self.participante = participante
			self.tiempo_medido = datetime.utcfromtimestamp(int(datos['tiempo_medido']))
			return True
		return False

	"""
	Comprobar el tiempo minimo de medicion del evento deportivo
	"""
	def comprobar_tiempo_minimo_medicion(self):
		evento = EventoDeportivo.objects.get(pk=self.evento)
		tiempo_comienzo = evento.instante_comienzo.replace(tzinfo=None)
		diferencia_tiempo = self.tiempo_medido - tiempo_comienzo

		if diferencia_tiempo.seconds/60 >= evento.tiempo_minimo_medicion:
			return True
		return False

	"""
	Asignar el punto de control al envio de cronomoetraje en base a la categoria del participante,
	el recorrido realizado por esa categoria, y los puntos de control de dicho recorrido.
	El punto de control de dicho recorrido que tenga configurado el equipo de cronometraje
	será el que corresponda al envío de cronometraje.
	"""
	def establecer_punto_control(self):
		puntos_de_control_recorrido = PuntoDeControl.objects.filter(recorrido=self.participante.categoria.recorrido.id)
		for punto_de_control in puntos_de_control_recorrido:
			if punto_de_control.equipo_de_cronometraje.id == self.equipo.id:
				self.punto_de_control = punto_de_control
				return True
		return False

	"""
	Comprobar que el envio no es una medida duplicada del paso del participante
	por un punto de control.
	En caso de que haya mas de una vuelta, se asigna el numero de vuelta
	"""
	def comprobar_vuelta(self):
		envios_crono_previos = EnvioCronometraje.objects.filter(
			evento=self.evento, 
			categoria=self.participante.categoria, 
			punto_de_control=self.punto_de_control, 
			participante=self.participante
		).order_by('-vuelta')

		if len(envios_crono_previos) == 0:
			return True

		elif len(envios_crono_previos) + 1 == self.participante.categoria.recorrido.numero_de_vueltas:
			tiempo_anterior = envios_crono_previos[0].tiempo_medido.replace(tzinfo=None)
			diferencia_tiempo = self.tiempo_medido - tiempo_anterior
			
			if diferencia_tiempo.seconds/60 >= self.participante.categoria.recorrido.tiempo_minimo_vuelta:
				self.vuelta = len(envios_crono_previos) + 1
				return True
			else: 
				return False

		else:
			return False


class EnvioDatosEstadoView(APIView):

	equipo = None

	def post(self, request, format=None):
		try:
			self.comprobar_api_key(request)

		except:
			return JsonResponse({"Resultado del envio": "NO PERMITIDO"})


		self.guardar_estado(request)
		return JsonResponse({"Resultado del envio": "OK"})


	"""
	Comprobar que el envio procede de un equipo de cronometraje válido,
	basandonos en su id y su api_key
	"""
	def comprobar_api_key(self, request):
		api_key = str(request.META.get('HTTP_API_KEY', ''))
		device_id = int(str(request.META.get('HTTP_USER_AGENT', ''))[7:])

		self.equipo = EquipoDeCronometraje.objects.get(pk=device_id)
		if api_key == self.equipo.api_key:
			return True
		return False

	def guardar_estado(self, request):
		datos = json.loads(request.body.decode("utf-8"))

		self.equipo.estado = bool(int(datos['estado']))
		self.equipo.estado_fecha = datetime.utcfromtimestamp(int(datos['estado_tiempo']))
		self.equipo.save()



"""---------------------------------------------------------------------------------
	
	Vistas pertenecientes al panel de administración de la aplicacion

-----------------------------------------------------------------------------------"""


class AdminInicioView(LoginRequiredMixin, TemplateView):
	template_name = 'cronoapp/admin/inicio.html'

	def get_context_data(self, **kargs):
		context = super(AdminInicioView, self).get_context_data(**kargs)
		context['eventos'] = EventoDeportivo.objects.filter(finalizado=False)
		context['equipos'] = EquipoDeCronometraje.objects.all()
		return context


### Equipos de cronometraje ###

class AdminEquipoListadoView(LoginRequiredMixin, ListView):
	model = EquipoDeCronometraje
	template_name = 'cronoapp/admin/equipo_listado.html'

class AdminEquipoDetalleView(LoginRequiredMixin, DetailView):
	model = EquipoDeCronometraje
	context_object_name = 'equipo'
	template_name = 'cronoapp/admin/equipo_detalle.html'

class AdminEquipoAnadirView(LoginRequiredMixin, CreateView):
	model = EquipoDeCronometraje
	form_class = EquipoDeCronometrajeFormulario
	template_name = 'cronoapp/admin/equipo_anadir.html'
	success_url = reverse_lazy('cronoapp:admin_equipo_listado')

	def form_valid(self, form):
		obj = form.save(commit=False)
		obj.api_key = self.generar_api_key()
		return super(AdminEquipoAnadirView, self).form_valid(form)

	def generar_api_key(self):
		key = hexlify(urandom(40))
		return key.decode()

class AdminEquipoEditarView(LoginRequiredMixin, UpdateView):
	model = EquipoDeCronometraje
	form_class = EquipoDeCronometrajeFormulario
	template_name = 'cronoapp/admin/equipo_editar.html'

class AdminEquipoEliminarView(LoginRequiredMixin, DeleteView):
	model = EquipoDeCronometraje
	template_name = 'cronoapp/admin/equipo_eliminar.html'
	success_url = reverse_lazy('cronoapp:admin_equipo_listado')


### Eventos deportivos ###

class AdminEventoListadoView(LoginRequiredMixin, TemplateView):
	def get_context_data(self, **kwargs):
		context = super(AdminEventoListadoView, self).get_context_data(**kwargs)
		context['eventos_finalizados'] = EventoDeportivo.objects.filter(finalizado=True)
		context['eventos_pendientes'] = EventoDeportivo.objects.filter(finalizado=False)
		return context

	template_name = 'cronoapp/admin/evento_listado.html'

class AdminEventoDetalleView(LoginRequiredMixin, DetailView):
	model = EventoDeportivo
	context_object_name = 'evento'
	template_name = 'cronoapp/admin/evento_detalle.html'

class AdminEventoAnadirView(LoginRequiredMixin, CreateView):
	model = EventoDeportivo
	form_class = EventoDeportivoFormulario
	template_name = 'cronoapp/admin/evento_anadir.html'

class AdminEventoEditarView(LoginRequiredMixin, UpdateView):
	model = EventoDeportivo
	context_object_name = 'evento'
	form_class = EventoDeportivoFormulario
	template_name = 'cronoapp/admin/evento_editar.html'

class AdminEventoEliminarView(LoginRequiredMixin, DeleteView):
	model = EventoDeportivo
	context_object_name = 'evento'
	template_name = 'cronoapp/admin/evento_eliminar.html'
	success_url = reverse_lazy('cronoapp:admin_evento_listado')

class AdminEventoComenzarView(LoginRequiredMixin, View):
	def get(self, request, *args, **kwargs):
		evento = get_object_or_404(EventoDeportivo, pk=self.kwargs['pk'])

		equipos_de_cronometraje_en_uso = []
		equipos_de_cronometraje_evento = []

		recorridos = Recorrido.objects.filter(evento=evento.id)
		if recorridos:
			for recorrido in recorridos:
				puntos_de_control = PuntoDeControl.objects.filter(recorrido=recorrido.id)
				if puntos_de_control:
					for punto_de_control in puntos_de_control:
						equipos_de_cronometraje_evento.append(punto_de_control.equipo_de_cronometraje)


		eventos_en_directo = EventoDeportivo.objects.filter(comenzado=True, finalizado=False)
		if eventos_en_directo:
			for evento_en_directo in eventos_en_directo:
				recorridos = Recorrido.objects.filter(evento=evento_en_directo.id)
				if recorridos:
					for recorrido in recorridos:
						puntos_de_control = PuntoDeControl.objects.filter(recorrido=recorrido.id)
						if puntos_de_control:
							for punto_de_control in puntos_de_control:
								equipos_de_cronometraje_en_uso.append(punto_de_control.equipo_de_cronometraje)

		equipos_en_comun = [i for i in equipos_de_cronometraje_evento if i in equipos_de_cronometraje_en_uso]
		if not equipos_en_comun:
			messages.add_message(request, messages.SUCCESS, 'El evento ha comenzado')
			evento.comenzar_evento()
		else:
			messages.add_message(request, messages.ERROR, 'El evento no ha comenzado, debido a que un equipo de cronometraje configurado para este evento se encuentra en uso en otro evento deportivo que se está llevando a cabo en estos momentos.')
		return HttpResponseRedirect(reverse_lazy('cronoapp:admin_evento_detalle', kwargs={'pk': self.kwargs['pk']}))

class AdminEventoFinalizarView(LoginRequiredMixin, View):
	def get(self, request, *args, **kwargs):
		evento = get_object_or_404(EventoDeportivo, pk=self.kwargs['pk'])
		if evento.comenzado == True and evento.finalizado == False:
			evento.finalizar_evento()
		return HttpResponseRedirect(reverse_lazy('cronoapp:admin_evento_detalle', kwargs={'pk': self.kwargs['pk']}))


### Recorridos ###

class AdminRecorridoListadoView(LoginRequiredMixin, ListView):
	model = Recorrido
	context_object_name = 'recorridos'
	template_name = 'cronoapp/admin/recorrido_listado.html'

	def get_context_data(self, **kwargs):
		context = super().get_context_data(**kwargs)
		context['recorridos'] = Recorrido.objects.filter(evento=self.kwargs['evento_id'])
		context['evento'] = EventoDeportivo.objects.get(pk=self.kwargs['evento_id'])
		return context

class AdminRecorridoDetalleView(LoginRequiredMixin, DetailView):
	model = Recorrido
	context_object_name = 'recorrido'
	template_name = 'cronoapp/admin/recorrido_detalle.html'

	def get_context_data(self, **kwargs):
		context = super().get_context_data(**kwargs)
		context['evento'] = EventoDeportivo.objects.get(pk=self.kwargs['evento_id'])
		return context

class AdminRecorridoAnadirView(LoginRequiredMixin, CreateView):
	model = Recorrido
	form_class = RecorridoFormulario
	template_name = 'cronoapp/admin/recorrido_anadir.html'

	def get_context_data(self, **kwargs):
		context = super().get_context_data(**kwargs)
		context['evento'] = EventoDeportivo.objects.get(pk=self.kwargs['evento_id'])
		return context

	def form_valid(self, form):
		obj = form.save(commit=False)
		obj.evento_id = self.kwargs['evento_id']
		return super(AdminRecorridoAnadirView, self).form_valid(form)

	def get_success_url(self):
	 	return reverse_lazy('cronoapp:admin_recorrido_listado', kwargs={'evento_id': self.kwargs['evento_id']})

class AdminRecorridoEditarView(LoginRequiredMixin, UpdateView):
	model = Recorrido
	form_class = RecorridoFormulario
	context_object_name = 'recorrido'
	template_name = 'cronoapp/admin/recorrido_editar.html'

	def get_context_data(self, **kwargs):
		context = super().get_context_data(**kwargs)
		context['evento'] = EventoDeportivo.objects.get(pk=self.kwargs['evento_id'])
		return context

	def get_success_url(self):
	 	return reverse_lazy('cronoapp:admin_recorrido_detalle', kwargs={
	 		'evento_id': self.kwargs['evento_id'],
	 		'pk': self.kwargs['pk']
	 	})

class AdminRecorridoEliminarView(LoginRequiredMixin, DeleteView):
	model = Recorrido
	context_object_name = 'recorrido'
	template_name = 'cronoapp/admin/recorrido_eliminar.html'

	def get_context_data(self, **kwargs):
		context = super().get_context_data(**kwargs)
		context['evento'] = EventoDeportivo.objects.get(pk=self.kwargs['evento_id'])
		return context

	def get_success_url(self):
	 	return reverse_lazy('cronoapp:admin_recorrido_listado', kwargs={'evento_id': self.kwargs['evento_id']})


### Puntos de Control ###

class AdminPuntoDeControlDetalleView(LoginRequiredMixin, DetailView):
	model = PuntoDeControl
	template_name = 'cronoapp/admin/puntocontrol_detalle.html'
	context_object_name = 'puntocontrol'

	def get_context_data(self, **kwargs):
		context = super().get_context_data(**kwargs)
		context['evento'] = EventoDeportivo.objects.get(pk=self.kwargs['evento_id'])
		context['recorrido'] = Recorrido.objects.get(pk=self.kwargs['recorrido_id'])
		return context

class AdminPuntoDeControlAnadirView(LoginRequiredMixin, CreateView):
	model = PuntoDeControl
	form_class = PuntoDeControlFormulario
	template_name = 'cronoapp/admin/puntocontrol_anadir.html'

	def get_form_kwargs(self, **kwargs):
		form_kwargs = super(AdminPuntoDeControlAnadirView, self).get_form_kwargs(**kwargs)
		form_kwargs["recorrido_id"] = self.kwargs['recorrido_id']
		form_kwargs["punto_de_control_id"] = None
		return form_kwargs

	def get_context_data(self, **kwargs):
		context = super().get_context_data(**kwargs)
		context['evento'] = EventoDeportivo.objects.get(pk=self.kwargs['evento_id'])
		context['recorrido'] = Recorrido.objects.get(pk=self.kwargs['recorrido_id'])
		return context

	def form_valid(self, form):
		obj = form.save(commit=False)
		obj.recorrido_id = self.kwargs['recorrido_id']
		return super(AdminPuntoDeControlAnadirView, self).form_valid(form)

	def get_success_url(self):
	 	return reverse_lazy('cronoapp:admin_recorrido_detalle', kwargs={
	 		'evento_id': self.kwargs['evento_id'],
	 		'pk': self.kwargs['recorrido_id'],
	 	})

class AdminPuntoDeControlEditarView(LoginRequiredMixin, UpdateView):
	model = PuntoDeControl
	form_class = PuntoDeControlFormulario
	context_object_name = 'puntocontrol'
	template_name = 'cronoapp/admin/puntocontrol_editar.html'

	def get_form_kwargs(self, **kwargs):
		form_kwargs = super(AdminPuntoDeControlEditarView, self).get_form_kwargs(**kwargs)
		form_kwargs["recorrido_id"] = self.kwargs['recorrido_id']
		form_kwargs["punto_de_control_id"] = self.kwargs['pk']
		return form_kwargs

	def get_context_data(self, **kwargs):
		context = super().get_context_data(**kwargs)
		context['evento'] = EventoDeportivo.objects.get(pk=self.kwargs['evento_id'])
		context['recorrido'] = Recorrido.objects.get(pk=self.kwargs['recorrido_id'])
		return context

	def get_success_url(self):
	 	return reverse_lazy('cronoapp:admin_puntocontrol_detalle', kwargs={
	 		'evento_id': self.kwargs['evento_id'],
	 		'recorrido_id': self.kwargs['recorrido_id'],
	 		'pk': self.kwargs['pk']
	 	})

class AdminPuntoDeControlEliminarView(LoginRequiredMixin, DeleteView):
	model = PuntoDeControl
	template_name = 'cronoapp/admin/puntocontrol_eliminar.html'
	context_object_name = 'puntocontrol'

	def get_context_data(self, **kwargs):
		context = super().get_context_data(**kwargs)
		context['evento'] = EventoDeportivo.objects.get(pk=self.kwargs['evento_id'])
		context['recorrido'] = Recorrido.objects.get(pk=self.kwargs['recorrido_id'])
		return context

	def get_success_url(self):
	 	return reverse_lazy('cronoapp:admin_recorrido_detalle', kwargs={
	 		'evento_id': self.kwargs['evento_id'],
	 		'pk': self.kwargs['recorrido_id'],
	 	})

### Categorias ###

class AdminCategoriaListadoView(LoginRequiredMixin, ListView):
	model = Categoria
	template_name = 'cronoapp/admin/categoria_listado.html'
	context_object_name = 'categorias'

	def get_context_data(self, **kwargs):
		context = super().get_context_data(**kwargs)
		context['categorias'] = Categoria.objects.filter(evento=self.kwargs['evento_id'])
		context['evento'] = EventoDeportivo.objects.get(pk=self.kwargs['evento_id'])
		return context

class AdminCategoriaDetalleView(LoginRequiredMixin, DetailView):
	model = Categoria
	template_name = 'cronoapp/admin/categoria_detalle.html'
	context_object_name = 'categoria'

	def get_context_data(self, **kwargs):
		context = super().get_context_data(**kwargs)
		context['evento'] = EventoDeportivo.objects.get(pk=self.kwargs['evento_id'])
		return context

class AdminCategoriaAnadirView(LoginRequiredMixin, CreateView):
	model = Categoria
	form_class = CategoriaFormulario
	template_name = 'cronoapp/admin/categoria_anadir.html'

	def get_form_kwargs(self, **kwargs):
		form_kwargs = super(AdminCategoriaAnadirView, self).get_form_kwargs(**kwargs)
		form_kwargs["evento_id"] = self.kwargs['evento_id']
		return form_kwargs

	def get_context_data(self, **kwargs):
		context = super().get_context_data(**kwargs)
		context['evento'] = EventoDeportivo.objects.get(pk=self.kwargs['evento_id'])
		return context

	def form_valid(self, form):
		obj = form.save(commit=False)
		obj.evento_id = self.kwargs['evento_id']
		return super(AdminCategoriaAnadirView, self).form_valid(form)

	def get_success_url(self):
	 	return reverse_lazy('cronoapp:admin_categoria_listado', kwargs={'evento_id': self.kwargs['evento_id']})

class AdminCategoriaEditarView(LoginRequiredMixin, UpdateView):
	model = Categoria
	form_class = CategoriaFormulario
	context_object_name = 'categoria'
	template_name = 'cronoapp/admin/categoria_editar.html'

	def get_form_kwargs(self, **kwargs):
		form_kwargs = super(AdminCategoriaEditarView, self).get_form_kwargs(**kwargs)
		form_kwargs["evento_id"] = self.kwargs['evento_id']
		return form_kwargs

	def get_context_data(self, **kwargs):
		context = super().get_context_data(**kwargs)
		context['evento'] = EventoDeportivo.objects.get(pk=self.kwargs['evento_id'])
		return context

	def get_success_url(self):
	 	return reverse_lazy('cronoapp:admin_categoria_detalle', kwargs={
	 		'evento_id': self.kwargs['evento_id'],
	 		'pk': self.kwargs['pk']
	 	})

class AdminCategoriaEliminarView(LoginRequiredMixin, DeleteView):
	model = Categoria
	template_name = 'cronoapp/admin/categoria_eliminar.html'
	context_object_name = 'categoria'

	def get_context_data(self, **kwargs):
		context = super().get_context_data(**kwargs)
		context['evento'] = EventoDeportivo.objects.get(pk=self.kwargs['evento_id'])
		return context

	def get_success_url(self):
	 	return reverse_lazy('cronoapp:admin_categoria_listado', kwargs={'evento_id': self.kwargs['evento_id']})


### Participantes ###

class AdminParticipanteListadoView(LoginRequiredMixin, ListView):
	model = Participante
	template_name = 'cronoapp/admin/participante_listado.html'
	context_object_name = 'participantes'

	def get_context_data(self, **kwargs):
		context = super().get_context_data(**kwargs)
		context['participantes'] = Participante.objects.filter(evento=self.kwargs['evento_id'])
		context['evento'] = EventoDeportivo.objects.get(pk=self.kwargs['evento_id'])
		return context

class AdminParticipanteDetalleView(LoginRequiredMixin, DetailView):
	model = Participante
	template_name = 'cronoapp/admin/participante_detalle.html'
	context_object_name = 'participante'

	def get_context_data(self, **kwargs):
		context = super().get_context_data(**kwargs)
		context['evento'] = EventoDeportivo.objects.get(pk=self.kwargs['evento_id'])
		return context

class AdminParticipanteAnadirView(LoginRequiredMixin, CreateView):
	model = Participante
	form_class = ParticipanteFormulario
	template_name = 'cronoapp/admin/participante_anadir.html'

	def get_form_kwargs(self, **kwargs):
		form_kwargs = super(AdminParticipanteAnadirView, self).get_form_kwargs(**kwargs)
		form_kwargs["evento_id"] = self.kwargs['evento_id']
		form_kwargs["participante_id"] = None
		return form_kwargs

	def get_context_data(self, **kwargs):
		context = super().get_context_data(**kwargs)
		context['evento'] = EventoDeportivo.objects.get(pk=self.kwargs['evento_id'])
		return context

	def form_valid(self, form):
		obj = form.save(commit=False)
		obj.evento_id = self.kwargs['evento_id']
		return super(AdminParticipanteAnadirView, self).form_valid(form)

	def get_success_url(self):
	 	return reverse_lazy('cronoapp:admin_participante_listado', kwargs={'evento_id': self.kwargs['evento_id']})

class AdminParticipanteEditarView(LoginRequiredMixin, UpdateView):
	model = Participante
	form_class = ParticipanteFormulario
	context_object_name = 'participante'
	template_name = 'cronoapp/admin/participante_editar.html'

	def get_form_kwargs(self, **kwargs):
		form_kwargs = super(AdminParticipanteEditarView, self).get_form_kwargs(**kwargs)
		form_kwargs["evento_id"] = self.kwargs['evento_id']
		form_kwargs["participante_id"] = self.kwargs['pk']
		return form_kwargs

	def get_context_data(self, **kwargs):
		context = super().get_context_data(**kwargs)
		context['evento'] = EventoDeportivo.objects.get(pk=self.kwargs['evento_id'])
		return context

	def get_success_url(self):
	 	return reverse_lazy('cronoapp:admin_participante_detalle', kwargs={
	 		'evento_id': self.kwargs['evento_id'],
	 		'pk': self.kwargs['pk']
	 	})

class AdminParticipanteEliminarView(LoginRequiredMixin, DeleteView):
	model = Participante
	template_name = 'cronoapp/admin/participante_eliminar.html'
	context_object_name = 'participante'

	def get_context_data(self, **kwargs):
		context = super().get_context_data(**kwargs)
		context['evento'] = EventoDeportivo.objects.get(pk=self.kwargs['evento_id'])
		return context

	def get_success_url(self):
	 	return reverse_lazy('cronoapp:admin_participante_listado', kwargs={'evento_id': self.kwargs['evento_id']})


class AdminObtenerEquiposCronometraje(LoginRequiredMixin, View):
	def get(self, request):
		data = []
		equipos = EquipoDeCronometraje.objects.all()
		for equipo in equipos:
			data.append({
				"id": equipo.pk,
				"nombre": equipo.nombre,
			})
		return JsonResponse(data, safe=False)


class AdminActualizarEquipoModoFecha(LoginRequiredMixin, View):
	def get(self, request):
		equipo_id = request.GET.get('equipo_id')
		equipo = EquipoDeCronometraje.objects.get(pk=equipo_id)
		equipo.modo_etiqueta_participante_fecha = datetime.utcnow()
		equipo.save()
		data = {}
		return JsonResponse(data)

class AdminObtenerEtiquetaParticipante(LoginRequiredMixin, View):
	def get(self, request):
		equipo_id = request.GET.get('equipo_id')
		try:
			envio = EnvioEtiquetaParticipante.objects.filter(equipo_id=equipo_id).latest('creado_fecha')
			data = {
				'etiqueta_rfid': envio.etiqueta_rfid,
			}
			EnvioEtiquetaParticipante.objects.filter(equipo_id=equipo_id).delete()
		except:
			data = {}
		return JsonResponse(data)


### Clasificacion ###

class AdminClasificacionView(LoginRequiredMixin, TemplateView):
	template_name = 'cronoapp/admin/clasificacion.html'

	def get_context_data(self, **kwargs):
		context = super().get_context_data(**kwargs)
		context['evento'] = EventoDeportivo.objects.get(pk=self.kwargs['evento_id'])
		return context